package Client;

import CampusInfo.*;

public class Controller
{
    public UniClient uniClient;

    public String u_id="";
    public String u_password="";
    public String u_fname="";
    public String u_lname="";
    public String u_email="";

    public boolean logged=false;

    public ClientFrame mainFrame;
    public String itemColumn[]={"Item Name","Category","Price","Posted Date"};
    public ItemTableModel itemTableModel = new ItemTableModel();

    public Controller(UniClient uniClient)
    {
        this.uniClient=uniClient;
    }

    public void setMainFrame(ClientFrame mainFrame)
    {
        this.mainFrame=mainFrame;
    }

    public void register(String u_id, String u_fname, String u_lname, String u_password, String u_email)
    {
        try
        {
            logged=uniClient.uniCommerceImpl.register(u_id, u_fname, u_lname, u_password, u_email);

            if(logged)
            {
                logged = false;
                mainFrame.showMessage("User register successfully","Please log in now");
            }
            else
            {
                mainFrame.showMessage("User register failed for unknow reason","You are lucky, this is a bug");
            }
        }
        catch(CampusInfo.ex_UidAlreadyExist ex)
        {
            mainFrame.showMessage("User ID already exist","Please chooce another loging ID");
        }
        catch(CampusInfo.ex_DatabaseOpFailed ex)
        {
            mainFrame.showMessage("Remote database operation failed","Please try another day");
        }
    }

    public void login(String u_id, String u_password)
    {
        try
        {
            User user=uniClient.uniCommerceImpl.login(u_id, u_password);

            if(user != null)
            {
                logged = true;
                this.u_id = u_id;
                this.u_password = u_password;
                this.u_fname = user.f_name;
                this.u_lname = user.l_name;
                this.u_email = user.u_email;

                mainFrame.updateView();
            }
            else
            {
                logged = false;
                mainFrame.showMessage("User login failed for unknow reason","You are lucky, this is a bug");
            }
        }
        catch(CampusInfo.ex_UidNotExist ex)
        {
            mainFrame.showMessage("User ID does not exist","Please register first");
        }
        catch(CampusInfo.ex_PwdNotMatch ex)
        {
            mainFrame.showMessage("User ID and Password do not match","Please verify and try again");
        }
        catch(CampusInfo.ex_DatabaseOpFailed ex)
        {
            mainFrame.showMessage("Remote database operation failed","Please try another day");
        }
    }

    public void logout()
    {
        this.logged=false;

        if(uniClient.uniCommerceImpl.logout(u_id))
        {
            mainFrame.showMessage("User logout successfully","");
        }
        else
        {
            mainFrame.showMessage("User has already logged out","You are lucky, this is a bug");
        }

        mainFrame.updateView();
    }

    public void update(String u_new_password, String u_new_fname, String u_new_lname, String u_new_email)
    {
        logged = false;

        try
        {
            if(uniClient.uniCommerceImpl.updateInfo(u_id, u_new_fname, u_new_lname, u_new_password, u_new_email))
            {
                mainFrame.showMessage("User update information successfully","Please log in now to validate change");
            }
            else
            {
                mainFrame.showMessage("User update failed for unknow reason","You are lucky, this is a bug");
            }
        }
        catch(CampusInfo.ex_AuthenticationFailed ex)
        {
            mainFrame.showMessage("User update failed for pass authentication","User login info lost, this is a bug");
        }
        catch(CampusInfo.ex_DatabaseOpFailed ex)
        {
            mainFrame.showMessage("Remote database operation failed","Please try another day");
        }

        mainFrame.updateView();
    }

    public void sellItem(String i_category, String i_name, float i_price, String i_note)
    {
        try
        {
            if(uniClient.uniCommerceImpl.sell(u_id, i_category, i_name, i_price, i_note))
            {
                mainFrame.showMessage("User sell item successfully","Please browse to view updated information");
            }
            else
            {
                mainFrame.showMessage("User sell failed for unknow reason","You are lucky, this is a bug");
            }
        }
        catch(CampusInfo.ex_AuthenticationFailed ex)
        {
            mainFrame.showMessage("User sell failed for pass authentication","User login info lost, this is a bug");
        }
        catch(CampusInfo.ex_DatabaseOpFailed ex)
        {
            mainFrame.showMessage("Remote database operation failed","Please try another day");
        }
    }

    public void buyItem(int i_id)
    {
        try
        {
            if(uniClient.uniCommerceImpl.buy(u_id, i_id))
            {
                mainFrame.showMessage("User buy item successfully","Please browse to view updated information");
            }
            else
            {
                mainFrame.showMessage("User buy failed for unknow reason","You are lucky, this is a bug");
            }
        }
        catch(CampusInfo.ex_AuthenticationFailed ex)
        {
            mainFrame.showMessage("User buy failed for pass authentication","User login info lost, this is a bug");
        }
        catch(CampusInfo.ex_DatabaseOpFailed ex)
        {
            mainFrame.showMessage("Remote database operation failed","Please try another day");
        }
        catch(CampusInfo.ex_ItemNotAvailableForBuy ex)
        {
            mainFrame.showMessage("User buy item failed - already sold","Please select another one or update list");
        }
    }

    public void sellCancel(int i_id)
    {
        try
        {
            if(uniClient.uniCommerceImpl.sellCancel(u_id, i_id))
            {
                mainFrame.showMessage("User cancel item sell successfully","Please browse to view updated information");
            }
            else
            {
                mainFrame.showMessage("User cancel item sell failed for unknow reason","You are lucky, this is a bug");
            }
        }
        catch(CampusInfo.ex_AuthenticationFailed ex)
        {
            mainFrame.showMessage("User cancel sell failed for pass authentication","User login info lost, this is a bug");
        }
        catch(CampusInfo.ex_DatabaseOpFailed ex)
        {
            mainFrame.showMessage("Remote database operation failed","Please try another day");
        }
        catch(CampusInfo.ex_ItemNotAvailableForCancel ex)
        {
            mainFrame.showMessage("User cancel sell failed - already sold or canceled","Please select another one or update list");
        }
        catch(CampusInfo.ex_ItemNotBelongToUser ex)
        {
            mainFrame.showMessage("User cancel sell failed - you do not own this item","Please select another one belong to you");
        }
    }

    public String[] history()
    {
        try
        {
            History [] historyList;
            historyList = uniClient.uniCommerceImpl.history(u_id);

            String [] historyItem = new String [historyList.length];

            for( int i=0; i<historyItem.length; i++)
            {
                historyItem[i] = historyList[i].h_datetime +": ";
                historyItem[i] += historyList[i].h_type +" [";
                historyItem[i] += historyList[i].i_category +"] ";
                historyItem[i] += historyList[i].i_name +" @ ";
                historyItem[i] += String.valueOf(((int)(historyList[i].i_price*100))/(float)100);
            }

            return historyItem;
        }
        catch(CampusInfo.ex_AuthenticationFailed ex)
        {
            mainFrame.showMessage("User view history failed for pass authentication","User login info lost, this is a bug");
            return new String[0];
        }
        catch(CampusInfo.ex_DatabaseOpFailed ex)
        {
            mainFrame.showMessage("Remote database operation failed","Please try another day");
            return new String[0];
        }
    }

    public void search(String i_category, String i_name)
    {
        try
        {
            Item[] itemList;
            itemList = uniClient.uniCommerceImpl.search(i_category, i_name);

            String data[][]=new String[itemList.length][6];

            for (int i = 0; i < itemList.length; i++)
            {
                data[i][0]=itemList[i].i_category;
                data[i][1]=itemList[i].i_name;
                data[i][2]=String.valueOf(((int)(itemList[i].i_price*100))/(float)100);
                data[i][3]=itemList[i].i_datetime;
                data[i][4]=itemList[i].i_note;
                data[i][5]=String.valueOf(itemList[i].i_id);
            }

            itemTableModel.setData(data);

            mainFrame.updateView();
        }
        catch (CampusInfo.ex_ItemNotFound ex)
        {
            mainFrame.showMessage("No items match your search criteria","Please refine your search statement");
        }
        catch (CampusInfo.ex_DatabaseOpFailed ex)
        {
            mainFrame.showMessage("Remote database operation failed","Please try another day");
       }
    }
}