package Server;

import CampusInfo.*;
import java.sql.*;
import java.util.*;

public class UniCommerceImpl extends UniCommercePOA
{
    private UniServer uniServer;
    private Hashtable userTable;

    public UniCommerceImpl(UniServer uniServer)
    {
        this.uniServer = uniServer;
        this.userTable=new Hashtable();
    }

    public boolean register(String u_id, String u_fname, String u_lname, String u_password, String u_email)
            throws CampusInfo.ex_DatabaseOpFailed, CampusInfo.ex_UidAlreadyExist
    {
        int returnID;

        try
        {
            String sql = "sp_register '"+u_id+"','"+u_fname+"','"+u_lname+"','"+u_password+"','"+u_email+"'";
            ResultSet rset = uniServer.stmt.executeQuery(sql);

            returnID = getReturnID(rset);
        }
        catch(Exception ex)
        {
            uniServer.frame.output("ERRO","Server","SQL server operation failed");
            ex.printStackTrace(System.out);
            throw new CampusInfo.ex_DatabaseOpFailed();
        }

        switch(returnID)
        {
            case 0:
                uniServer.frame.output("OKAY","Register","User "+u_id+" register successfully");
                return true;
            case 1:
                uniServer.frame.output("OKAY","Register","User "+u_id+" register fail due to existing u_id");
                throw new CampusInfo.ex_UidAlreadyExist();
            default:
                return false;
        }
    }

    public User login(String u_id, String u_password)
            throws CampusInfo.ex_DatabaseOpFailed, CampusInfo.ex_UidNotExist, CampusInfo.ex_PwdNotMatch
    {
        int returnID;

        try
        {
            String sql = "sp_login '"+u_id+"','"+u_password+"'";
            ResultSet rset = uniServer.stmt.executeQuery(sql);

            returnID = getReturnID(rset);
        }
        catch(Exception ex)
        {
            uniServer.frame.output("ERRO","Server","SQL server operation failed");
            ex.printStackTrace(System.out);
            throw new CampusInfo.ex_DatabaseOpFailed();
        }

        switch(returnID)
        {
            case 0:
                uniServer.frame.output("OKAY","Login","User "+u_id+" login successfully");
                break;
            case 1:
                uniServer.frame.output("OKAY","Login","User "+u_id+" login ID does not existed");
                throw new CampusInfo.ex_UidNotExist();
            case 2:
                uniServer.frame.output("OKAY","Login","User "+u_id+" login ID and password does not match");
                throw new CampusInfo.ex_PwdNotMatch();
            default:
                return null;
        }

        try
        {
            String sql = "sp_readinfo '"+u_id+"','"+u_password+"'";
            ResultSet rset = uniServer.stmt.executeQuery(sql);
            rset.next();
            User user = new User(rset.getString(1),rset.getString(2),rset.getString(3));
            userTable.put(u_id,u_password);
            uniServer.frame.output("OKAY","Login","User "+u_id+" info read successfully");
            return user;
        }
        catch(Exception ex)
        {
            uniServer.frame.output("ERRO","Server","SQL server operation failed");
            ex.printStackTrace(System.out);
            throw new CampusInfo.ex_DatabaseOpFailed();
        }
    }

    public boolean logout(String u_id)
    {
        boolean isLogged = userTable.containsKey(u_id);
        if( isLogged) userTable.remove(u_id);

        uniServer.frame.output("OKAY","Logout","User "+u_id+" logged out");
        return isLogged;
    }

    public boolean updateInfo(String u_id, String u_new_fname, String u_new_lname, String u_new_password, String u_new_email)
            throws CampusInfo.ex_DatabaseOpFailed, CampusInfo.ex_AuthenticationFailed
    {
        int returnID;

        if( !userTable.containsKey(u_id))
        {
            uniServer.frame.output("ERRO","UpdateInfo","User "+u_id+" is not in logged status");
            throw new CampusInfo.ex_AuthenticationFailed();
        }

        String u_password = (String) userTable.get(u_id);

        try
        {
            String sql = "sp_updateinfo '"+u_id+"','"+u_password+"','"+u_new_fname+"','"+u_new_lname+"','"+u_new_password+"','"+u_new_email+"'";
            ResultSet rset = uniServer.stmt.executeQuery(sql);

            returnID = getReturnID(rset);
        }
        catch(Exception ex)
        {
            uniServer.frame.output("ERRO","Server","SQL server operation failed");
            ex.printStackTrace(System.out);
            throw new CampusInfo.ex_DatabaseOpFailed();
        }

        switch(returnID)
        {
            case 0:
                uniServer.frame.output("OKAY","UpdateInfo","User "+u_id+" update info successfully");
                return true;
            case 1:
                uniServer.frame.output("ERRO","UpdateInfo","User "+u_id+" update info failed in passed authentication");
                throw new CampusInfo.ex_AuthenticationFailed();
            default:
                return false;
        }
    }

    public boolean sell(String u_id, String i_category, String i_name, float i_price, String i_note)
            throws CampusInfo.ex_DatabaseOpFailed, CampusInfo.ex_AuthenticationFailed
    {
        int returnID;

        if( !userTable.containsKey(u_id))
        {
            uniServer.frame.output("ERRO","Sell","User "+u_id+" is not in logged status");
            throw new CampusInfo.ex_AuthenticationFailed();
        }

        String u_password = (String) userTable.get(u_id);

        try
        {
            String sql = "sp_sell '"+u_id+"','"+u_password+"','"+i_category+"','"+i_name+"','"+i_price+"','"+i_note+"'";
            ResultSet rset = uniServer.stmt.executeQuery(sql);

            returnID = getReturnID(rset);
        }
        catch(Exception ex)
        {
            uniServer.frame.output("ERRO","Server","SQL server operation failed");
            ex.printStackTrace(System.out);
            throw new CampusInfo.ex_DatabaseOpFailed();
        }

        switch(returnID)
        {
            case 0:
                uniServer.frame.output("OKAY","Sell","User "+u_id+" sell a new item successfully");
                return true;
            case 1:
                uniServer.frame.output("ERRO","Sell","User "+u_id+" update info failed in passed authentication");
                throw new CampusInfo.ex_AuthenticationFailed();
            default:
                return false;
        }
    }

    public boolean buy(String u_id, int i_id)
            throws CampusInfo.ex_DatabaseOpFailed, CampusInfo.ex_AuthenticationFailed, CampusInfo.ex_ItemNotAvailableForBuy
    {
        int returnID;

        if( !userTable.containsKey(u_id))
        {
            uniServer.frame.output("ERRO","Buy","User "+u_id+" is not in logged status");
            throw new CampusInfo.ex_AuthenticationFailed();
        }

        String u_password = (String) userTable.get(u_id);

        try
        {
            String sql = "sp_buy '"+u_id+"','"+u_password+"','"+i_id+"'";
            ResultSet rset = uniServer.stmt.executeQuery(sql);

            returnID = getReturnID(rset);
        }
        catch(Exception ex)
        {
            uniServer.frame.output("ERRO","Server","SQL server operation failed");
            ex.printStackTrace(System.out);
            throw new CampusInfo.ex_DatabaseOpFailed();
        }

        switch(returnID)
        {
            case 0:
                uniServer.frame.output("OKAY","Buy","User "+u_id+" buy a item successfully");
                return true;
            case 1:
                uniServer.frame.output("ERRO","Buy","User "+u_id+" buy a item failed in passed authentication");
                throw new CampusInfo.ex_AuthenticationFailed();
            case 2:
                uniServer.frame.output("WARN","Buy","User "+u_id+" buy a item failed cos already sold");
                throw new CampusInfo.ex_ItemNotAvailableForBuy();
            default:
                return false;
        }
    }

    public boolean sellCancel(String u_id, int i_id)
            throws CampusInfo.ex_DatabaseOpFailed, CampusInfo.ex_AuthenticationFailed, CampusInfo.ex_ItemNotAvailableForCancel, CampusInfo.ex_ItemNotBelongToUser
    {
        int returnID;

        if( !userTable.containsKey(u_id))
        {
            uniServer.frame.output("ERRO","SellCancel","User "+u_id+" is not in logged status");
            throw new CampusInfo.ex_AuthenticationFailed();
        }

        String u_password = (String) userTable.get(u_id);

        try
        {
            String sql = "sp_sellcancel '"+u_id+"','"+u_password+"','"+i_id+"'";
            ResultSet rset = uniServer.stmt.executeQuery(sql);

            returnID = getReturnID(rset);
        }
        catch(Exception ex)
        {
            uniServer.frame.output("ERRO","Server","SQL server operation failed");
            ex.printStackTrace(System.out);
            throw new CampusInfo.ex_DatabaseOpFailed();
        }

        switch(returnID)
        {
            case 0:
                uniServer.frame.output("OKAY","SellCancel","User "+u_id+" cancel a item successfully");
                return true;
            case 1:
                uniServer.frame.output("ERRO","SellCancel","User "+u_id+" cancel a item failed in passed authentication");
                throw new CampusInfo.ex_AuthenticationFailed();
            case 2:
                uniServer.frame.output("WARN","SellCancel","User "+u_id+" cancel a item failed cos already sold");
                throw new CampusInfo.ex_ItemNotAvailableForCancel();
            case 3:
                uniServer.frame.output("WARN","SellCancel","User "+u_id+" cancel a item failed cos not invalid ownership");
                throw new CampusInfo.ex_ItemNotBelongToUser();
            default:
                return false;
        }
    }

    public History[] history(String u_id)
            throws CampusInfo.ex_DatabaseOpFailed, CampusInfo.ex_AuthenticationFailed
    {
        int colNum;
        Vector temp=new Vector();

        if( !userTable.containsKey(u_id))
        {
            uniServer.frame.output("ERRO","History","User "+u_id+" is not in logged status");
            throw new CampusInfo.ex_AuthenticationFailed();
        }

        String u_password = (String) userTable.get(u_id);

        try
        {
            String sql = "sp_history '"+u_id+"','"+u_password+"'";
            ResultSet rset = uniServer.stmt.executeQuery(sql);

            colNum = rset.getMetaData().getColumnCount();

            if(colNum != 1)
            {
                while (rset.next())
                {
                    History item = new History();
                    item.i_id = rset.getInt(1);
                    switch(rset.getInt(2))
                    {
                        case 0:
                            item.h_type = "Sell";
                            break;
                        case 1:
                            item.h_type = "Buy";
                            break;
                        case 2:
                            item.h_type = "Sell Cancel";
                            break;
                        default:
                            item.h_type = "Unkown";
                            break;
                    }
                    item.h_datetime = rset.getDate(3).toString();
                    item.i_category = rset.getString(4);
                    item.i_name = rset.getString(5);
                    item.i_price = rset.getFloat(6);
                    temp.add(item);
                }
            }
        }
        catch(Exception ex)
        {
            uniServer.frame.output("ERRO","Server","SQL server operation failed");
            ex.printStackTrace(System.out);
            throw new CampusInfo.ex_DatabaseOpFailed();
        }

        if(colNum == 1)
        {
            uniServer.frame.output("ERRO","History","User "+u_id+" view history failed in passed authentication");
            throw new CampusInfo.ex_AuthenticationFailed();
        }

        History [] historyList=new History[temp.size()];

        for(int i=0; i<historyList.length; i++) historyList[i] = (History) temp.elementAt(i);

        return historyList;
    }

    public Item[] search(String i_category, String i_name)
            throws CampusInfo.ex_DatabaseOpFailed, CampusInfo.ex_ItemNotFound
    {
        Item[] itemList;
        Vector temp=new Vector();

        try
        {
            String sql = "sp_search '"+i_category+"','"+i_name+"'";
            ResultSet rset = uniServer.stmt.executeQuery(sql);

            while (rset.next())
            {
                Item item=new Item(rset.getInt(1),rset.getString(2),rset.getString(3),rset.getFloat(4),rset.getDate(5).toString(),rset.getString(6));
                temp.add(item);
            }

            uniServer.frame.output("OKAY","Search","Search operation successfully");
        }
        catch(Exception ex)
        {
            uniServer.frame.output("ERRO","Server","SQL server operation failed");
            ex.printStackTrace(System.out);
            throw new CampusInfo.ex_DatabaseOpFailed();
        }

        if (temp.size() == 0) throw new CampusInfo.ex_ItemNotFound();
        else itemList=new Item[temp.size()];

        for(int i=0; i<itemList.length; i++) itemList[i] = (Item) temp.elementAt(i);

        return itemList;
    }

    private int getReturnID(ResultSet rset)
            throws Exception
    {
        int colNum = rset.getMetaData().getColumnCount();
        if(colNum == 1 && rset.next ()) return rset.getInt(1);
        return -1;
    }
}